Module interval

Interval implementations.

Structs

EmptyRangeError
Trying to make empty interval.
IncorrectRangeError
Trying to make interval from range where start > end.
Interval
Describes not empty interval start..=end.

Enums

NewWithLenError
Trying to make zero len or out of bounds interval.
TryFromRangeError
Trying to make interval from range where start > end or empty range.